@@ -11,7 +11,7 @@ the main public interface here.
 '''
 import os, pwd
 from xml.dom.minidom import parse, Document, Node
-from core import pycore
+from core.netns import nodes
 from core.api import coreapi
 
 def addelementsfromlist(dom, parent, iterable, name, attr_name):
@@ -33,6 +33,13 @@ def addtextelementsfromlist(dom, parent, iterable, name, attrs):
     ''' XML helper to iterate through a list and add items to parent using tags
     of the given name, attributes specified in the attrs tuple, and having the 
     text of the item within the tags.
+    Example: addtextelementsfromlist(dom, parent, ('a','b','c'), "letter",
+                                     (('show','True'),))
+    <parent>
+      <letter show="True">a</letter>
+      <letter show="True">b</letter>
+      <letter show="True">c</letter>
+    </parent>
     '''
     for item in iterable:
         element = dom.createElement(name)
@@ -42,6 +49,28 @@ def addtextelementsfromlist(dom, parent, iterable, name, attrs):
         txt = dom.createTextNode(item)
         element.appendChild(txt)
 
+def addtextelementsfromtuples(dom, parent, iterable, attrs=()):
+    ''' XML helper to iterate through a list of tuples and add items to
+    parent using tags named for the first tuple element,
+    attributes specified in the attrs tuple, and having the 
+    text of second tuple element.
+    Example: addtextelementsfromtuples(dom, parent,
+                 (('first','a'),('second','b'),('third','c')),
+                 (('show','True'),))
+    <parent>
+      <first show="True">a</first>
+      <second show="True">b</second>
+      <third show="True">c</third>
+    </parent>
+    '''
+    for name, value in iterable:
+        element = dom.createElement(name)
+        for k,v in attrs:
+            element.setAttribute(k, v)
+        parent.appendChild(element)
+        txt = dom.createTextNode(value)
+        element.appendChild(txt)
+
 def gettextelementstolist(parent):
     ''' XML helper to parse child text nodes from the given parent and return
     a list of (key, value) tuples.
@@ -84,6 +113,26 @@ def addtextparamtoparent(dom, parent, name, value):
     txt = dom.createTextNode(value)
     p.appendChild(txt)
     return p
+    
+def addparamlisttoparent(dom, parent, name, values):
+    ''' XML helper to return a parameter list and optionally add it to the
+    parent element:
+    <paramlist name="name">
+       <item value="123">
+       <item value="456">
+    </paramlist>
+    '''
+    if values is None:
+        return None
+    p = dom.createElement("paramlist")
+    if parent:
+        parent.appendChild(p)
+    p.setAttribute("name", name)
+    for v in values:
+        item = dom.createElement("item")
+        item.setAttribute("value", str(v))
+        p.appendChild(item)
+    return p
  
 def getoneelement(dom, name):
     e = dom.getElementsByTagName(name)
@@ -113,19 +162,22 @@ def getparamssetattrs(dom, param_names, target):
             setattr(target, param_name, str(value))
 
 def xmltypetonodeclass(session, type):
-    ''' Helper to convert from a type string to a class name in pycore.nodes.*.
+    ''' Helper to convert from a type string to a class name in nodes.*.
     '''
-    if hasattr(pycore.nodes, type):
-        return eval("pycore.nodes.%s" % type)
+    if hasattr(nodes, type):
+        return eval("nodes.%s" % type)
     else:
         return None
 
 class CoreDocumentParser(object):
-    def __init__(self, session, filename):
+    def __init__(self, session, filename, start=False,
+                 nodecls=nodes.CoreNode):
         self.session = session
         self.verbose = self.session.getcfgitembool('verbose', False)
         self.filename = filename
         self.dom = parse(filename)
+        self.start = start
+        self.nodecls = nodecls
         
         #self.scenario = getoneelement(self.dom, "Scenario")
         self.np = getoneelement(self.dom, "NetworkPlan")
@@ -139,6 +191,8 @@ class CoreDocumentParser(object):
         # link parameters parsed in parsenets(), applied in parsenodes()
         self.linkparams = {}
         
+        self.parsedefaultservices()
+        self.parseorigin()
         self.parsenets()
         self.parsenodes()
         self.parseservices()
@@ -197,7 +251,7 @@ class CoreDocumentParser(object):
                           (name, type))
                 continue
             n = self.session.addobj(cls = nodecls, objid = id, name = name,
-                                    start = False)
+                                    start = self.start)
             if name in self.coords:
                 x, y, z = self.coords[name]
                 n.setposition(x, y, z)
@@ -207,27 +261,40 @@ class CoreDocumentParser(object):
             # links between two nets (e.g. switch-switch)
             for ifc in net.getElementsByTagName("interface"):
                 netid = str(ifc.getAttribute("net"))
-                linkednets.append((n, netid))
+                ifcname = str(ifc.getAttribute("name"))
+                linkednets.append((n, netid, ifcname))
             self.parsemodels(net, n)
         # link networks together now that they all have been parsed
-        for (n, netid) in linkednets:
+        for (n, netid, ifcname) in linkednets:
             try:
                 n2 = n.session.objbyname(netid)
             except KeyError:
                 n.warn("skipping net %s interface: unknown net %s" % \
                           (n.name, netid))
                 continue
-            n.linknet(n2)
+            upstream = False
+            netif = n.getlinknetif(n2)
+            if netif is None:
+                netif = n2.linknet(n)
+            else:
+                netif.swapparams('_params_up')
+                upstream = True
+            key = (n2.name,  ifcname)
+            if key in self.linkparams:
+                for (k, v) in self.linkparams[key]:
+                    netif.setparam(k, v)
+            if upstream:
+                netif.swapparams('_params_up')
     
     def parsenodes(self):
         for node in self.np.getElementsByTagName("Node"):
             id, name, type = self.getcommonattributes(node)
             if type == "rj45":
-                nodecls = pycore.nodes.RJ45Node
+                nodecls = nodes.RJ45Node
             else:
-                nodecls = pycore.nodes.CoreNode
+                nodecls = self.nodecls
             n = self.session.addobj(cls = nodecls, objid = id, name = name,
-                                    start = False)
+                                    start = self.start)
             if name in self.coords:
                 x, y, z = self.coords[name]
                 n.setposition(x, y, z)
@@ -334,7 +401,50 @@ class CoreDocumentParser(object):
         else:
             value = int(value)
         return (key, value)
-    
+
+    def parseorigin(self):
+        ''' Parse any origin tag from the Mobility Plan and set the CoreLocation
+            reference point appropriately.
+        '''
+        origin = getoneelement(self.mp, "origin")
+        if not origin:
+            return
+        location = self.session.location
+        geo = []
+        attrs = ("lat","lon","alt")
+        for i in xrange(3):
+            a = origin.getAttribute(attrs[i])
+            if a is not None:
+                a = float(a)
+            geo.append(a)
+        location.setrefgeo(geo[0], geo[1], geo[2])
+        scale = origin.getAttribute("scale100")
+        if scale is not None:
+            location.refscale = float(scale)
+        point = getoneelement(origin, "point")
+        if point is not None and point.firstChild is not None:
+            xyz = point.firstChild.nodeValue.split(',')
+            if len(xyz) == 2:
+                xyz.append('0.0')
+            if len(xyz) == 3:
+                xyz = map(lambda(x): float(x), xyz)
+                location.refxyz = (xyz[0], xyz[1], xyz[2])
+            
+    def parsedefaultservices(self):
+        ''' Prior to parsing nodes, use session.services manager to store
+        default services for node types
+        '''
+        for node in self.sp.getElementsByTagName("Node"):
+            type = node.getAttribute("type")
+            if type == '':
+                continue # node-specific service config
+            services = []
+            for service in node.getElementsByTagName("Service"):
+                services.append(str(service.getAttribute("name")))
+            self.session.services.defaultservices[type] = services
+            self.session.info("default services for type %s set to %s" % \
+                              (type, services))
+        
     def parseservices(self):
         ''' After node objects exist, parse service customizations and add them
         to the nodes.
@@ -343,6 +453,8 @@ class CoreDocumentParser(object):
         # parse services and store configs into session.services.configs
         for node in self.sp.getElementsByTagName("Node"):
             name = node.getAttribute("name")
+            if name == '':
+                continue # node type without name
             n = self.session.objbyname(name)
             if n is None:
                 self.warn("skipping service config for unknown node '%s'" % \
@@ -355,6 +467,15 @@ class CoreDocumentParser(object):
                         svclists[n.objid] += "|" + svcname
                     else:
                         svclists[n.objid] = svcname
+        # nodes in NetworkPlan but not in ServicePlan use the 
+        # default services for their type
+        for node in self.np.getElementsByTagName("Node"):
+            id, name, type = self.getcommonattributes(node)
+            if id in svclists:
+                continue # custom config exists
+            else:
+                svclists[int(id)] = None # use defaults
+
         # associate nodes with services
         for objid in sorted(svclists.keys()):
             n = self.session.obj(objid)
@@ -482,6 +603,8 @@ class CoreDocumentWriter(Document):
 
     def populatefromsession(self):
         self.session.emane.setup() # not during runtime?
+        self.addorigin()
+        self.adddefaultservices()
         self.addnets()
         self.addnodes()
         self.addmetadata()
@@ -502,7 +625,7 @@ class CoreDocumentWriter(Document):
         '''
         with self.session._objslock:
             for net in self.session.objs():
-                if not isinstance(net, pycore.nodes.PyCoreNet):
+                if not isinstance(net, nodes.PyCoreNet):
                     continue
                 self.addnet(net)
 
@@ -533,15 +656,27 @@ class CoreDocumentWriter(Document):
         parameters. TODO: Interface parameters should be moved to the model
         construct, then this separate method shouldn't be required.
         '''
-        if not hasattr(netif, "node") or netif.node is None:
-            return
         params = netif.getparams()
         if len(params) == 0:
             return
         model = self.createElement("model")
         model.setAttribute("name", "netem")
         model.setAttribute("netif", netif.name)
-        model.setAttribute("peer", netif.node.name)
+        if hasattr(netif, "node") and netif.node is not None:
+            model.setAttribute("peer", netif.node.name)
+        # link between switches uses one veth interface
+        elif hasattr(netif, "othernet") and netif.othernet is not None:
+            if netif.othernet.name == n.getAttribute("name"):
+                model.setAttribute("peer", netif.net.name)
+            else:
+                model.setAttribute("peer", netif.othernet.name)
+                model.setAttribute("netif", netif.localname)
+            # hack used for upstream parameters for link between switches
+            # (see LxBrNet.linknet())
+            if netif.othernet.objid == int(n.getAttribute("id")):
+                netif.swapparams('_params_up')
+                params = netif.getparams()
+                netif.swapparams('_params_up')
         has_params = False
         for k, v in params:
             # default netem parameters are 0 or None
@@ -580,7 +715,7 @@ class CoreDocumentWriter(Document):
         '''
         with self.session._objslock:
             for node in self.session.objs():
-                if not isinstance(node, pycore.nodes.PyCoreNode):
+                if not isinstance(node, nodes.PyCoreNode):
                     continue
                 self.addnode(node)
 
@@ -631,12 +766,13 @@ class CoreDocumentWriter(Document):
         for ifc in net.netifs(sort=True):
             if not hasattr(ifc, "othernet") or not ifc.othernet:
                 continue
-            if net.objid == ifc.net.objid:
-                continue
             i = self.createElement("interface")
             n.appendChild(i)
-            i.setAttribute("name", ifc.name)
-            if ifc.net:
+            if net.objid == ifc.net.objid:
+                i.setAttribute("name", ifc.localname)
+                i.setAttribute("net", ifc.othernet.name)
+            else:
+                i.setAttribute("name", ifc.name)
                 i.setAttribute("net", ifc.net.name)
 
     def addposition(self, node):
@@ -664,6 +800,47 @@ class CoreDocumentWriter(Document):
         coords = self.createTextNode(coordstxt)
         pt.appendChild(coords)
 
+    def addorigin(self):
+        ''' Add origin to Motion Plan using canvas reference point.
+            The CoreLocation class maintains this reference point.
+        '''
+        refgeo = self.session.location.refgeo
+        origin = self.createElement("origin")
+        attrs = ("lat","lon","alt")
+        have_origin = False
+        for i in xrange(3):
+            if refgeo[i] is not None:
+                origin.setAttribute(attrs[i], str(refgeo[i]))
+                have_origin = True
+        if not have_origin:
+            return
+        if self.session.location.refscale != 1.0: # 100 pixels = refscale m
+            origin.setAttribute("scale100", str(self.session.location.refscale))
+        if self.session.location.refxyz != (0.0, 0.0, 0.0):
+            pt = self.createElement("point")
+            origin.appendChild(pt)
+            x,y,z = self.session.location.refxyz
+            coordstxt = "%s,%s" % (x,y)
+            if z:
+                coordstxt += ",%s" % z
+            coords = self.createTextNode(coordstxt)
+            pt.appendChild(coords)
+
+        self.mp.appendChild(origin)
+        
+    def adddefaultservices(self):
+        ''' Add default services and node types to the ServicePlan.
+        '''
+        for type in self.session.services.defaultservices:
+            defaults = self.session.services.getdefaultservices(type)
+            spn = self.createElement("Node")
+            spn.setAttribute("type", type)
+            self.sp.appendChild(spn)
+            for svc in defaults:
+                s = self.createElement("Service")
+                spn.appendChild(s)
+                s.setAttribute("name", str(svc._name))
+        
     def addservices(self, node):
         ''' Add services and their customizations to the ServicePlan.
         '''
@@ -763,10 +940,15 @@ class CoreDocumentWriter(Document):
             addtextparamtoparent(self, meta, k, v)
             #addparamtoparent(self, meta, k, v)
 
-def opensessionxml(session, filename):
+def opensessionxml(session, filename, start=False, nodecls=nodes.CoreNode):
     ''' Import a session from the EmulationScript XML format.
     '''
-    doc = CoreDocumentParser(session, filename)
+    doc = CoreDocumentParser(session, filename, start, nodecls)
+    if start:
+        session.name = os.path.basename(filename)
+        session.filename = filename
+        session.node_count = str(session.getnodecount())
+        session.instantiate()
 
 def savesessionxml(session, filename):
     ''' Export a session to the EmulationScript XML format.
