@@ -20,7 +20,7 @@ setup(name = "corens3-python",
                      glob.glob("examples/*.py")),
                     ],
       description = "Python ns-3 components of CORE",
-      url = "http://cs.itd.nrl.navy.mil/work/core/",
+      url = "http://www.nrl.navy.mil/itd/ncs/products/core",
       author = "Boeing Research & Technology",
       author_email = "core-dev@pf.itd.nrl.navy.mil",
       license = "GPLv2",
