@@ -22,7 +22,7 @@ setup(name = "core-python-netns",
           "Linux network namespaces",
       data_files = [("sbin", ('vcmd', 'vnoded', 'netns')), ],
       ext_modules = [netns, vcmd],
-      url = "http://cs.itd.nrl.navy.mil/work/core/",
+      url = "http://www.nrl.navy.mil/itd/ncs/products/core",
       author = "Boeing Research & Technology",
       author_email = "core-dev@pf.itd.nrl.navy.mil",
       license = "BSD",
