@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2013 the Boeing Company.
+# Copyright 2005-2014 the Boeing Company.
 # See the LICENSE file included in this distribution.
 #
 
@@ -240,7 +240,10 @@ bind . <Control-s> "fileSaveDialogBox {}"
 
 .menubar.file add separator
 .menubar.file add command -label "Export Python script..." -command exportPython
-.menubar.file add command -label "Execute Python script..." -command execPython
+.menubar.file add command -label "Execute XML or Python script..." \
+	-command { execPython false }
+.menubar.file add command -label "Execute Python script with options..." \
+	-command { execPython true }
 
 .menubar.file add separator
 .menubar.file add command -label "Open current file in editor..." \
@@ -293,7 +296,7 @@ bind . <Control-s> "fileSaveDialogBox {}"
     }
 .menubar.file add separator
 foreach f $g_mrulist {
-    .menubar.file add command -label "$f" -command "mrufile \"$f\""
+    .menubar.file add command -label "$f" -command "mrufile {$f}"
 }
 # end Boeing changes 
 .menubar.file add separator
@@ -595,9 +598,9 @@ menu .menubar.session -tearoff 1
 #
 menu .menubar.help -tearoff 0
 .menubar.help add command -label "Online manual (www)" -command \
-  "_launchBrowser http://pf.itd.nrl.navy.mil/core/core-html/"
+  "_launchBrowser http://downloads.pf.itd.nrl.navy.mil/docs/core/core-html/"
 .menubar.help add command -label "CORE website (www)" -command \
-  "_launchBrowser http://cs.itd.nrl.navy.mil/work/core/index.php"
+  "_launchBrowser http://www.nrl.navy.mil/itd/ncs/products/core"
 .menubar.help add command -label "Mailing list (www)" -command \
   "_launchBrowser http://pf.itd.nrl.navy.mil/mailman/listinfo/core-users"
 .menubar.help add command -label "About" -command popupAbout
